@@ -135,9 +135,11 @@ Nothing.
 
 =head1 AUTHOR
 
-Copyright (c) XAO, Inc.
+Copyright (c) 2003-2005 Andrew Maltsev
 
-Marcos Alves <alves@xao.com>.
+Copyright (c) 2001-2003 Marcos Alves, XAO Inc.
+
+<am@ejelta.com> -- http://ejelta.com/xao/
 
 =head1 SEE ALSO
 
@@ -155,7 +157,7 @@ use XAO::Objects;
 use base XAO::Objects->load(objname => 'Web::Page');
 
 use vars qw($VERSION);
-($VERSION)=(q$Id: Search.pm,v 1.5 2002/12/12 17:54:20 am Exp $ =~ /(\d+\.\d+)/);
+$VERSION=(0+sprintf('%u.%03u',(q$Id: Search.pm,v 2.1 2005/01/14 01:39:57 am Exp $ =~ /\s(\d+)\.(\d+)\s/))) || die "Bad VERSION";
 
 ###############################################################################
 sub display ($;%)
